/*
 * Capability mapping between tasks.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/consts.h>
#include <l4/sys/err.h>
#include <l4/sys/ipc.h>
#include <l4/sys/task.h>

#include "map.h"



/* Map a capability to another task. */

long ipc_map_capability(l4_cap_idx_t task, struct ipc_mapped_cap mapped_cap)
{
  return l4_error(l4_task_map(task, L4RE_THIS_TASK_CAP,
                              l4_obj_fpage(mapped_cap.cap, 0, mapped_cap.rights),
                              l4_map_obj_control(mapped_cap.mapped_cap, L4_MAP_ITEM_MAP) |
                                mapped_cap.obj_rights));
}

/* Map several capabilities to another task. */

long ipc_map_capabilities(l4_cap_idx_t task, struct ipc_mapped_cap mapped_caps[],
                          unsigned int *count)
{
  long err = L4_EOK;
  unsigned int i;

  for (i = 0; l4_is_valid_cap(mapped_caps[i].cap) && !err; i++)
    err = ipc_map_capability(task, mapped_caps[i]);

  if (count != NULL)
    *count = i;

  return err;
}

// vim: tabstop=2 expandtab shiftwidth=2
